local tldr = CreateFrame("Frame")
local tldr_options = {}

SLASH_TLDR1 = "/tldr"

tldr:SetScript('OnEvent', function(self, event, ...) self[event](self, ...) end)

tldr:RegisterEvent("PLAYER_LOGIN")
tldr:RegisterEvent("LOOT_READY")
tldr:RegisterEvent("GOSSIP_SHOW")
tldr:RegisterEvent("QUEST_COMPLETE")
tldr:RegisterEvent("QUEST_DETAIL")
tldr:RegisterEvent("QUEST_GREETING")
tldr:RegisterEvent("QUEST_PROGRESS")
tldr:RegisterEvent("TALKINGHEAD_REQUESTED")

SlashCmdList["TLDR"] = function (msg, editbox)
	if msg == '' then
		InterfaceOptionsFrame_OpenToCategory(tldr_options.panel)
	else
		print "TL;DR - Unknown command."
	end
end

function tldr:LOOT_READY()
	if autoCloseLoot == false then return end
	shiftState = IsShiftKeyDown()
	if invertShift == true then
		shiftState = not shiftState
	end
	
	if shiftState == false then
		LootFrame:Close()
	end
end

function tldr:TALKINGHEAD_REQUESTED()
	if disableTalkingHead == true then TalkingHeadFrame:Hide() end
end

function tldr:PLAYER_LOGIN()
	tldr_options.panel = CreateFrame( "Frame", "tldr_optionsPanel", UIParent )
 	tldr_options.panel.name = "TL;DR"
 	InterfaceOptions_AddCategory(tldr_options.panel)

	local title = tldr_options.panel:CreateFontString(nil, "ARTWORK", "GameFontNormalLarge")
	title:SetPoint("TOPLEFT", 16, -16)
	title:SetText("Too Long; Didn't Read")

	tldr_disableVerbose_cb = CreateFrame("CheckButton", "tldr_disableVerbose_cb", tldr_options.panel, "ChatConfigCheckButtonTemplate")
	tldr_disableVerbose_cb:SetPoint("TOPLEFT", 5, -35)
	tldr_disableVerbose_cbText:SetText("Disable verbose mode")
	tldr_disableVerbose_cb.tooltip = "Checking this will disable chat notifications when TL;DR performs an action."
	tldr_disableVerbose_cb:SetScript("OnClick",fnDisableVB)

	tldr_disableAutoAccept_cb = CreateFrame("CheckButton", "tldr_disableAutoAccept_cb", tldr_options.panel, "ChatConfigCheckButtonTemplate")
	tldr_disableAutoAccept_cb:SetPoint("TOPLEFT", 5, -55)
	tldr_disableAutoAccept_cbText:SetText("Disable auto accepting quests")
	tldr_disableAutoAccept_cb.tooltip = "Checking this will disable TL;DR from accepting new quests on your behalf."
	tldr_disableAutoAccept_cb:SetScript("OnClick",fnDisableAutoAccept)

	tldr_invertShift_cb = CreateFrame("CheckButton", "tldr_invertShift_cb", tldr_options.panel, "ChatConfigCheckButtonTemplate")
	tldr_invertShift_cb:SetPoint("TOPLEFT", 5, -75)
	tldr_invertShift_cbText:SetText("Invert Shift enable/disable functionality")
	tldr_invertShift_cb.tooltip = "Invert shift key functionality (shift enables addon rather than disabling)"
	tldr_invertShift_cb:SetScript("OnClick",fnInvertShift)

	tldr_auto_close_loot_window_cb = CreateFrame("CheckButton", "tldr_auto_close_loot_window_cb", tldr_options.panel, "ChatConfigCheckButtonTemplate")
	tldr_auto_close_loot_window_cb:SetPoint("TOPLEFT", 5, -95)
	tldr_auto_close_loot_window_cbText:SetText("Auto-close loot window")
	tldr_auto_close_loot_window_cb.tooltip = "Automatically closes the loot window immediately. With auto-loot enabled in the options, the loot window doesn't appear with this checked."
	tldr_auto_close_loot_window_cb:SetScript("OnClick",fnAutoCloseLoot)
	
	tldr_disable_talking_head_cb = CreateFrame("CheckButton", "tldr_disable_talking_head_cb", tldr_options.panel, "ChatConfigCheckButtonTemplate")
	tldr_disable_talking_head_cb:SetPoint("TOPLEFT", 5, -115)
	tldr_disable_talking_head_cbText:SetText("Disable talking heads")
	tldr_disable_talking_head_cb.tooltip = "Disable the talking heads that pop up for quests."
	tldr_disable_talking_head_cb:SetScript("OnClick",fnTalkingHead)

	tldr_gossip_bt = CreateFrame("Button", "tldr_gossip_bt", GossipFrame.NineSlice,"UIPanelButtonTemplate")
	tldr_gossip_bt:SetPoint("TOP",GossipFrame.NineSlice, "TOP", 0, -30)
	tldr_gossip_btText:SetText("TL;DR: Enabled")
	tldr_gossip_bt.tooltip = "Toggle enabling TL;DR on this NPC"
	tldr_gossip_bt:SetWidth(125)
	tldr_gossip_bt:SetScript("OnClick",fnToggleBlacklist)


	-- Set options values
	if invertShift == nil then invertShift = false end
	if disableVerbose == nil then disableVerbose = false end
	if disableAutoAccept == nil then disableAutoAccept = false end
	if autoCloseLoot == nil then autoCloseLoot = false end
	if disableTalkingHead == nil then disableTalkingHead = false end
	if tldrBlacklist == nil then tldrBlacklist = {} end
	tldr_invertShift_cb:SetChecked(invertShift)
	tldr_disableVerbose_cb:SetChecked(disableVerbose)
	tldr_disableAutoAccept_cb:SetChecked(disableAutoAccept)
	tldr_auto_close_loot_window_cb:SetChecked(autoCloseLoot)
	tldr_disable_talking_head_cb:SetChecked(disableTalkingHead)
	
end

function fnToggleBlacklist()
	local targetGUID
	targetType,_,_,_,_,targetGUID = strsplit("-", UnitGUID("target"))
	if targetType == "Player" then return end
	
	if tldrBlacklist[targetGUID] == true then
		tldrBlacklist[targetGUID] = false
		tldr_gossip_btText:SetText("TL;DR: Enabled")
	else
		tldrBlacklist[targetGUID] = true
		tldr_gossip_btText:SetText("TL;DR: Disabled")
	end
end

function tldr:GOSSIP_SHOW()
	--local status, questsitch

	status = C_GossipInfo:GetOptions()
	questsitch = C_GossipInfo:GetNumAvailableQuests() + C_GossipInfo:GetNumActiveQuests()
	shiftState = IsShiftKeyDown()
	ctrlState = IsControlKeyDown()
	
	if GetUnitName("target") == GossipFrameTitleText:GetText() and UnitIsPlayer("target") == false then
		tldr_gossip_bt:Show()
		_,_,_,_,_,the_id = strsplit("-", UnitGUID("target"))
		if tldrBlacklist[the_id] == true then
			if disableVerbose == false then print("TL;DR - Skipping blacklisted npc") end
			tldr_gossip_btText:SetText("TL;DR: Disabled")
			return
		else
			tldr_gossip_btText:SetText("TL;DR: Enabled")
		end
	else
		tldr_gossip_bt:Hide()
	end

	
	if invertShift == true then
		shiftState = not shiftState
	end
	
	GossipCount = 0
	optionCount = 0
	for _, the_type in ipairs(C_GossipInfo:GetOptions()) do
		optionCount = optionCount + 1
		if the_type.type == "gossip" then
			GossipCount = GossipCount + 1
			GossipSpot = optionCount
		end
	end
	
	
	if shiftState == true then return end

	myQuest = C_GossipInfo:GetOptions()
	for i=1, #myQuest do
		if string.sub(myQuest[i].name,1,10) == "|cFF0000FF" then
			C_GossipInfo.SelectOption(myQuest[i].gossipOptionID)
		end	
	end		

	if questsitch == 0 then
		intQuests = C_GossipInfo:GetActiveQuests()
		if (#C_GossipInfo:GetOptions()) == 1 then
			if disableVerbose == false then print("TL;DR - Selecting only available option.") end
			SelectGossipOption(1)
		end
	else
		myQuest = C_GossipInfo:GetActiveQuests()
		for i=1, C_GossipInfo:GetNumActiveQuests() do
			if myQuest[i].isComplete == true then
				C_GossipInfo.SelectActiveQuest(myQuest[i].questID)
			end	
		end	
		if C_GossipInfo:GetNumAvailableQuests() > 0 and disableAutoAccept == false then
			
			myQuest = C_GossipInfo:GetAvailableQuests()
			for i=1, C_GossipInfo:GetNumAvailableQuests() do
				if myQuest[i].isTrivial == false and myQuest[i].frequency ~= 3 then
					C_GossipInfo.SelectAvailableQuest(myQuest[i].questID)
				end
			end
		end
	end
end

function tldr:QUEST_GREETING()
	shiftState = IsShiftKeyDown()
	if invertShift == true then
		shiftState = not shiftState
	end
	if shiftState == false and disableAutoAccept == false then
		if disableVerbose == false then print("TL;DR - Accepted quest") end
		SelectAvailableQuest(1)
		for i=1, GetNumActiveQuests() do
			--_, _, isLowLevel, isCompleted = GetActiveQuests(i)
			_,isCompleted = GetActiveTitle(i)
			if isCompleted == true then
				SelectActiveQuest(i)
			end	
		end
	end
end

function tldr:QUEST_PROGRESS()
	shiftState = IsShiftKeyDown()
	if invertShift == true then
		shiftState = not shiftState
	end
	if shiftState == false then
		CompleteQuest(1)
	end
end
function tldr:QUEST_COMPLETE()
	shiftState = IsShiftKeyDown()
	if invertShift == true then
		shiftState = not shiftState
	end
	if shiftState == false and (GetNumQuestChoices() <= 1)then
		if disableVerbose == false then print("TL;DR - Completed quest.") end
		GetQuestReward(1)
	end
end

function tldr:QUEST_DETAIL()
	shiftState = IsShiftKeyDown()
	if invertShift == true then
		shiftState = not shiftState
	end
	if shiftState == false and disableAutoAccept == false then
		if disableVerbose == false then print("TL;DR - Accepted quest") end
		AcceptQuest()
	end
end

function fnDisableVB()
	disableVerbose = tldr_disableVerbose_cb:GetChecked()
end
function fnDisableAutoAccept()
	disableAutoAccept = tldr_disableAutoAccept_cb:GetChecked()
end
function fnInvertShift()
	invertShift = tldr_invertShift_cb:GetChecked()
end
function fnTalkingHead()
	disableTalkingHead = tldr_disable_talking_head_cb:GetChecked()
end
function fnAutoCloseLoot()
	autoCloseLoot = tldr_auto_close_loot_window_cb:GetChecked()
end
